/** @file SkChunkLoader.m
 SodaKit - Download rows in chunks via method=getRows
 
 Copyright (c) 2012 Edgemoor. All rights reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

#import "SkChunkLoader.h"
#import "SkLogger.h"
#import <RestKit/RestKit.h>

@implementation SkChunkLoader
//! Initialize with optional user supplied object
- (id)      initWithUserInfo: (NSObject*) userInfo_ {
    self = [super init];
    if ( self != nil ) {
        _userInfo = userInfo_;
        _eState   = eSkChunkLoaderState_Running;
    }
    return self;
}
//! NSObject.isEqual compares userInfo
- (BOOL)    isEqual: (id)   anObject_ {
    BOOL    equal = FALSE;
    if ( [anObject_ isKindOfClass: [SkChunkLoader class]] ) {
        SkChunkLoader* other_ = (SkChunkLoader*) anObject_;
        while ( self.userInfo != nil && other_.userInfo != nil ) {
            if ( [self.userInfo isEqual: other_.userInfo] ) {
                equal = TRUE;
                break;
            }
            if ( [self.userInfo   isKindOfClass: [NSURL class]] &&
                 [other_.userInfo isKindOfClass: [NSURL class]] ) {
                NSURL* selfURL = (NSURL*) self.userInfo;
                NSURL* othrURL = (NSURL*) other_.userInfo;
                if ( [selfURL isEqual: othrURL] ) {
                    equal = TRUE;
                    break;
                }
                
            }
            break;
        }
    }
    return equal;
}
#pragma mark - RKObjectLoaderDelegate

//! RKObjectLoaderDelegate - send when an object failed to load collection due to an error
- (void)    objectLoader: (RKObjectLoader*) objectLoader_ 
        didFailWithError: (NSError*)        error_ {
    if ( [objectLoader_ isEqual: _loader] ) {
        SKLogMWrn( @"NSError: %@ at %@", [error_ localizedDescription], SKLogM_THIS_METHOD );
        _eState = eSkChunkLoaderState_DoneFail;
        [[NSNotificationCenter defaultCenter] postNotificationName: [SkChunkLoader notifyName]
                                                            object: self];
    } else {
        SKLogMWrn( @"mismatch objectLoader_ / _loader at %@ - ignored", SKLogM_THIS_METHOD );
    }
    return;
}

//! RKObjectLoaderDelegate - Sent when object loader completed successfully
- (void)    objectLoader: (RKObjectLoader*) objectLoader_ 
           didLoadObject: (id)              object_ {
    if ( [objectLoader_ isEqual: _loader] ) {
        RKManagedObjectMapping*  mapping     = (RKManagedObjectMapping*) objectLoader_.objectMapping;
        NSEntityDescription*     entity      = mapping.entity;
        NSString*                entityName  = [entity name];
        if ( mapping.objectStore != nil ) {
            NSError* error = nil;
            [mapping.objectStore save: &error];
            if ( error != nil ) {
                SKLogMWrn( @"%@ save failed: %@", entityName, [error localizedDescription] );
                _eState = eSkChunkLoaderState_DoneFail;
            } else {
                _eState = eSkChunkLoaderState_DoneSuccess;
            }
        }
        [[NSNotificationCenter defaultCenter] postNotificationName: [SkChunkLoader notifyName]
                                                            object: self];
    } else {
        SKLogMWrn( @"mismatch objectLoader_ / _loader at %@ - ignored", SKLogM_THIS_METHOD );        
    }
    return;
}

//! RKObjectLoaderDelegate - Invoked just after parsing has completed, but before object mapping begins.
- (void)    objectLoader: (RKObjectLoader*) objectLoader_ 
             willMapData: (inout id*)       mappableData_ {
    if ( mappableData_ != nil && *mappableData_ != nil ) {
        NSObject*                data        = *mappableData_;
        RKManagedObjectMapping*  mapping     = (RKManagedObjectMapping*) objectLoader_.objectMapping;
        NSEntityDescription*     entity      = mapping.entity;
        NSString*                entityName  = [entity name];
                
        if ( [data isKindOfClass: [NSMutableArray class]] ) {
            NSMutableArray*   aData    = (NSMutableArray*) data;
            for ( NSUInteger  iData    = 0; iData < aData.count; iData++ ) {
                NSObject*     curValue = [aData objectAtIndex: iData];
                NSDictionary* newValue = [NSDictionary dictionaryWithObject: curValue forKey: entityName];
                [aData replaceObjectAtIndex: iData withObject: newValue];
            }
        }
    }
    return;
}

//! RKRequestDelegate - request is about to be prepared for sending to remote host
- (void)    requestWillPrepareForSend:  (RKRequest*) request_ {
    do {
        if ( ![request_ isKindOfClass: [RKObjectLoader class]] ) {
            SKLogMWrn( @"unexpected RKRequest at %@ - ignored", SKLogM_THIS_METHOD );
            break;
        }
        if ( _loader != nil ) {
            SKLogMWrn( @"unexpected _loader at %@ - discarded", SKLogM_THIS_METHOD );
            _loader = nil;
        }
        _loader = (RKObjectLoader*) request_;
        _eState = eSkChunkLoaderState_Running;
    } while ( false );
}

//! Dealloc - ensure that we release self as RKObjectLoader delegate
- (void)        dealloc {
    while ( _loader != nil ) {
        if ( _loader.delegate != nil ) {
            if ( [self isEqual: _loader.delegate] ) {
                _loader.delegate = nil;
            }
        }
        _loader = nil;
        break;
    }
}

//! NSNotificaton name
+ (NSString*)   notifyName {
    return @"SkChunkLoaderNotifyName";
}
@synthesize userInfo    = _userInfo;
@synthesize loader      = _loader;
@synthesize eState      = _eState;
@end
