/** @file SkLogger.m
 SodaKit - Logging macros and methods
 
 Copyright (c) 2012 Edgemoor. All rights reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
#import <Foundation/Foundation.h>
#import "SkLogger.h"

static const char* SKLogLevelPrefix[] = {"", ".", "..","..,","..,.","..,..","..,..,",
                                         "..,..,.","..,..,..","..,..,..,"};
static const int   SKLogLevelMax      = sizeof( SKLogLevelPrefix[0] ) / sizeof( SKLogLevelPrefix );

void SKLogArrayLevel(      NSArray*      array,      NSUInteger curLevel, NSUInteger maxLevel );
void SKLogDictionaryLevel( NSDictionary* dictionary, NSUInteger curLevel, NSUInteger maxLevel );

#pragma mark - private functions
//! Internal - Log a NSDictionary level ( recursive )
void    SKLogArrayLevel( NSArray* array, NSUInteger curLevel, NSUInteger maxLevel ) {
    NSUInteger  line    = 1;
    
    for ( NSObject* value in array ) {
        if ( [value isKindOfClass: [NSDictionary class]] ) {
            SKLogMInf( @"%s%2i NSDictionary",SKLogLevelPrefix[ curLevel ], line++ );
            SKLogDictionaryLevel( (NSDictionary*) value, curLevel +1, maxLevel );
            continue;
        }
        if ( [value isKindOfClass: [NSArray class]] ) {
            SKLogMInf( @"%s%2i NSArray",SKLogLevelPrefix[ curLevel ], line++ );
            SKLogArrayLevel( (NSArray*) value, curLevel +1, maxLevel );
            continue;
        }
        SKLogMInf( @"%s%2i %@",SKLogLevelPrefix[ curLevel ], line++, value );
    }
    
}


//! Internal - Log a NSDictionary level ( recursive )
void    SKLogDictionaryLevel( NSDictionary* dictionary, NSUInteger curLevel, NSUInteger maxLevel ) {
    NSArray*    keys = [[dictionary allKeys] sortedArrayUsingSelector: 
                                        @selector( caseInsensitiveCompare: )];
    NSObject*   value   = nil;
    NSUInteger  line    = 1;
    
    for ( NSString* key in keys ) {
        value = [dictionary objectForKey: key];
        if ( [value isKindOfClass: [NSDictionary class]] ) {
            SKLogMInf( @"%s%2i %@ - NSDictionary",SKLogLevelPrefix[ curLevel ], line++, key );
            SKLogDictionaryLevel( (NSDictionary*) value, curLevel +1, maxLevel );
            continue;
        }
        if ( [value isKindOfClass: [NSArray class]] ) {
            SKLogMInf( @"%s%2i %@ - NSArray",SKLogLevelPrefix[ curLevel ], line++, key );
            SKLogArrayLevel( (NSArray*) value, curLevel +1, maxLevel );
            continue;
        }
        SKLogMInf( @"%s%2i %@ - %@",SKLogLevelPrefix[ curLevel ], line++, key, value );
    }
}


#pragma mark - public functions
//! Log ( at Info ) a NSDictionary, with optional starting comment, and max depth ( if > 0 )
void    SKLogArray( NSArray* array, NSObject* title, NSInteger maxDepth ) {
    if ( true ) {
        NSUInteger maxLevel = SKLogLevelMax;
        if ( 0 < maxDepth && maxDepth < maxLevel ) {
            maxLevel = maxDepth;
        }
        SKLogMInf( @"---- NSArray %@ --------",title );
        SKLogArrayLevel( array,  1,  maxLevel );
        SKLogMInf( @"==== NSArray %@ ========",title );
        
    }
}
//! Log ( at Info ) a NSDictionary, with optional starting comment, and max depth ( if > 0 )
void    SKLogDictionary( NSDictionary* dictionary, NSObject* title, NSInteger maxDepth ) {
    if ( true ) {
        NSUInteger maxLevel = SKLogLevelMax;
        if ( 0 < maxDepth && maxDepth < maxLevel ) {
            maxLevel = maxDepth;
        }
        SKLogMInf( @"---- NSDictionary %@ --------",title );
        SKLogDictionaryLevel( dictionary,  1,  maxLevel );
        SKLogMInf( @"==== NSDictionary %@ ========",title );

    }
}
//! Log an NSError
void    SKLogError( NSError* error_, SEL source_, NSString* context_ ) {
    NSMutableString* buffer = [[NSMutableString alloc] initWithString: error_.localizedDescription];
    [buffer appendString: @"/n at: "];
    [buffer appendString: NSStringFromSelector( source_ )];
    if ( context_ != nil && 0 < context_.length ) {
        [buffer appendString: @"/n context: "];
        [buffer appendString: context_];
    }
    SKLogMErr( @"Exception: %@", buffer );
}//! Log an NSException
void    SKLogException( NSException* exception, SEL source, NSString* context ) {
    NSMutableString* buffer = [[NSMutableString alloc] initWithCapacity: 100];
    if ( exception != nil && exception.reason != nil ) {
        [buffer appendString: exception.reason];
    } else {
        [buffer appendString: @"unknown exception reason"];
    }
    [buffer appendString: @"/n at: "];
    if ( source != nil ) {
        [buffer appendString: NSStringFromSelector( source )];        
    }
    if ( context != nil && 0 < context.length ) {
        [buffer appendString: @"/n context: "];
        [buffer appendString: context];
    }
    SKLogMErr( @"Exception: %@", buffer );
}